# Copyright 2012-2014 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt

"""
Item
----
"""

from nive_datastore.i18n import _
from nive.container import Container
from nive.definitions import ObjectConf


class item(Container):

    def Init(self):
        self.queryRestraints = {}, {}
    
    


# Root definition ------------------------------------------------------------------
#@nive_module
configuration = ObjectConf(
    id = "item",
    context = "nive_datastore.item.item",
    extensions = ("nive_datastore.pydispatch.Dispatcher",),
    name = _("Data item"),
    description = ""
)



# -*- coding: utf-8 -*-

import unittest

from nive.helper import FormatConfTestFailure

from nive_datastore import app, root, item




class TestConf(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_conf1(self):
        r=app.configuration.test()
        if not r:
            return
        print(FormatConfTestFailure(r))
        self.assert_(False, "Configuration Error")

    def test_conf2(self):
        r=root.configuration.test()
        if not r:
            return
        print(FormatConfTestFailure(r))
        self.assert_(False, "Configuration Error")

    def test_conf3(self):
        r=item.configuration.test()
        if not r:
            return
        print(FormatConfTestFailure(r))
        self.assert_(False, "Configuration Error")



"""
use as import:
from nive_datastore.i18n import _
"""
from translationstring import TranslationStringFactory
_ = TranslationStringFactory('nive_datastore')

from nive.i18n import translator
from nive.i18n import translate



# -*- coding: utf-8 -*-

import time
import unittest

from nive.helper import FormatConfTestFailure

from nive_datastore.webapi import view




class TestConf(unittest.TestCase):

    def test_confcontainer(self):
        r=view.configuration.test()
        if not r:
            return
        print(FormatConfTestFailure(r))
        self.assert_(False, "Configuration Error")

    def test_confitems(self):
        r=view.localstorage_views.test()
        if not r:
            return
        print(FormatConfTestFailure(r))
        self.assert_(False, "Configuration Error")




# -*- coding: utf-8 -*-

import unittest

from nive.security import User
from nive.definitions import ContainmentError
from nive_datastore.app import DataStorage, IDataStorage
from nive_datastore.tests import db_app
from nive_datastore.tests import __local


class AppTest_db(object):

    def setUp(self):
        self._loadApp()
        self.remove = []
        
    def tearDown(self):
        u = User("test")
        u.groups.append("group:manager")
        root = self.app.root
        for r in self.remove:
            root.Delete(r, u)
        self.app.Close()

    def test_realdata(self):
        ccc = self.app.db.GetCountEntries()
        r = self.app.root
        user = User("test")
        user.groups.append("group:manager")
        # add to root
        o1 = db_app.create_bookmark(r, user)
        self.remove.append(o1.id)

        o2 = db_app.create_track(r, user)
        self.remove.append(o2.id)

        self.assertEqual(ccc+2, self.app.db.GetCountEntries(), "Delete failed")

        r.Delete(o1.id, user=user, obj=o1)
        r.Delete(o2.id, user=user)
        self.assertEqual(ccc, self.app.db.GetCountEntries(), "Delete failed")


    def test_realdata_container(self):
        ccc = self.app.db.GetCountEntries()
        r=self.app.root
        user = User("test")
        user.groups.append("group:manager")
        # add to root
        o1 = db_app.create_bookmark(r, user)
        self.remove.append(o1.id)

        o2 = db_app.create_track(r, user)
        self.remove.append(o2.id)

        self.assertEqual(ccc+2, self.app.db.GetCountEntries(), "Delete failed")

        o3 = db_app.create_track(o1, user)

        self.assertRaises(ContainmentError, db_app.create_bookmark, o2, user)

        self.assertEqual(ccc+3, self.app.db.GetCountEntries(), "Delete failed")

        r.Delete(o1.id, user=user, obj=o1)
        r.Delete(o2.id, user=user)
        self.assertEqual(ccc, self.app.db.GetCountEntries(), "Delete failed")


    def test_interfaces(self):
        self.assertFalse(IDataStorage.providedBy(123))
        self.assertTrue(IDataStorage.providedBy(DataStorage(None)))
        
        
    def test_app(self):
        app = DataStorage(None)
        


class AppTest_db_Sqlite(AppTest_db, __local.SqliteTestCase):
    pass
        
class AppTest_db_MySql(AppTest_db, __local.MySqlTestCase):
    pass
    
class AppTest_db_Postgres(AppTest_db, __local.PostgreSqlTestCase):
    pass
        


# -*- coding: utf-8 -*-
# Copyright 2012-2014 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt

"""
Nive Datastore example
----------------------
The `Nive DataStore` is a highlevel storage with a focus on build in 
functional extensions like form renderer, workflow integration, 
hierarchical data structures, flexible renderer and a web api.

Sqllite, Postgres or Mysql are used as data backend so the goal is not replace
database systems but to provide a flexible, easy and convenient way 
to store data items.

Database connection and basic datastore application configuration
::
    
    from nive.definitions import AppConf, DatabaseConf

    app = AppConf("nive_datastore.app",
                  id = "storage",
                  title = "My Data Storage",
                  maxStoreItems = 20,
                  maxBatchNumber = 100,
                  search = {"default": {
                               "type": "bookmark", 
                               "container": False,
                               "fields": ["id", "link", "comment", "pool_changedby"],
                               "parameter": {"share": True}
                  }},
                  subtree = {"default": {
                                 "levels": 0,              # the number of levels to include, 0=include all
                                 "descent": (IContainer,), # item types or interfaces to descent into subtree
                                 "fields": {},             # if empty uses type definition toJson defaults
                                 "parameter": {"pool_state": 1}
                  }},
                  defaultSubtree = "all"
    )
    dbConfiguration = DatabaseConf(
        fileRoot="/var/opt/datastore",
        context="Sqlite3",
        dbName="/var/opt/datastore/items.db"
    )
    app.modules.append(dbConfiguration)

**Items**

Item data field definitions are registered as typed collections. Each collection
has a unique type id, a custom set of data fields and many other options. Use
`nive.definitions.ObjectConf` as configuration class to define a new collection.

A simple example. Stores a bookmark and comment: ::

    bookmark = ObjectConf(
        id = "bookmark",
        name = _("Bookmarks"),
        dbparam = "bookmarks",
        data = (
          FieldConf(id="link",     datatype="url",  size=500,   default="",  name="Link url"),
          FieldConf(id="share",    datatype="bool", size=2,     default=False,name="Share link"),
          FieldConf(id="comment",  datatype="text", size=50000, default="",  name="Comment")
        ),
        forms = {
          "newItem": {"fields": ("link", "share", "comment") }, 
          "setItem": {"fields": ("link", "share", "comment") }
        },
        toJson = ("link", "share", "comment", "id", "pool_create", "pool_change"),
        template = "bookmark.pt",
    )
    app.modules.append(bookmark)  # app is the AppConf() defined in the previous example 
    
**Groups and permissions**

``configuration.acl`` and ``configuration.groups``

*Permissions:* read, search, add, edit, delete, webform, tojson, toxml, render, action

*Groups:* group:reader, group:manager, group:admin
    
By default groups and permissions are set up to reflect simple content management
workflows with four different roles. Groups can be used for both: view permissions
and workflow execution. 

The data storage has no workflow activated by default. So by reassigning permissions 
in `configuration.acl` the predefined groups can simply be customized. 

**Webforms**

The systems provides a build in form renderer. Form configurations are included in the collection
definition (`configuration.form`) and support by default create and edit modes. 

**Workflow**

Workflow processes can be added to the system to build a event system or extend the 
functionality. You can define and use multiple workflow processes and assign a process
to an item as needed (or define defaults). A workflow process is organized in states and
transitions. A transition can take one or multiple callbacks to be executed with the 
transition.  

By default the workflow is disabled. See `nive.definitions.AppConf.workflowEnabled` and 
`nive.definitions.ObjectConf.workflowID`

**Meta data**

The system automatically tracks create and change information in a shared meta table
(shared by all items). This table also stores the items globally unique id. To define 
database fields used for all collections use `nive.definitions.AppConf.meta`.

"""
import copy

from nive.definitions import implementer, Interface
from nive.definitions import AppConf, GroupConf
from nive.definitions import SystemFlds, UserFlds, WorkflowFlds
from nive.security import ALL_PERMISSIONS, Allow, Everyone, Deny
from nive.application import Application

#@nive_module
configuration = AppConf(
    id = "storage",
    title = "Nive Data Storage",
    context = "nive_datastore.app.DataStorage",
    workflowEnabled = True,
    meta = copy.deepcopy(list(SystemFlds)) + copy.deepcopy(list(UserFlds)) + copy.deepcopy(list(WorkflowFlds)),
    translations="nive_datastore:locale/"
)

configuration.modules = [
    # items / collections
    # -> no collections defined by default. See documentation how to define collections 
    #    based on 'nive.definitions.ObjectConf' 
    # root 
    "nive_datastore.root",
    # workflow
    #"nive_datastore.workflow",
    # web api (view layer)
    "nive_datastore.webapi",
    "nive_datastore.webapi.view.stringRendererConf",
    # extensions
    "nive.extensions.filename",
    "nive.extensions.localgroups",
    # tools
    "nive.tools.dbStructureUpdater", "nive.tools.cmsstatistics",
    "nive.tools.exportJson", "nive.tools.dbJsonDump",
    # administration and persistence
    "nive.components.adminview",
    "nive.extensions.persistence.dbPersistenceConfiguration"
]


configuration.acl = [
    (Allow, Everyone, 'view'),
    (Allow, 'group:admin', ALL_PERMISSIONS), 
    (Deny, Everyone, ALL_PERMISSIONS),
]

configuration.groups = [
    GroupConf(id="group:reader",  name="group:reader"),
    GroupConf(id="group:owner",   name="group:owner"),
    GroupConf(id="group:editor",  name="group:editor"),
    GroupConf(id="group:admin",   name="group:admin"),
]



class IDataStorage(Interface):
    """
    """

@implementer(IDataStorage)
class DataStorage(Application):
    """ the main cms application class """


    



# Copyright 2012-2014 Arndt Droullier, Nive GmbH. All rights reserved.
# Released under GPL3. See license.txt

"""
Root
----
The *root* is the entry point of the data storage. All contained items are stored 
in the database. The root itself does not store anything in the database by default.
The system supports multiple roots with unique urls to access items. 

Also this object provides search functions and sql query wrappers.
"""

from nive_datastore.i18n import _
from nive.container import Root
from nive.definitions import RootConf
from nive.definitions import AllTypesAllowed


class root(Root):

    def Init(self):
        self.queryRestraints = {}, {}
    
    


# Root definition ------------------------------------------------------------------
#@nive_module
configuration = RootConf(
    id = "api",
    context = "nive_datastore.root.root",
    default = True,
    subtypes = AllTypesAllowed,
    extensions = ("nive_datastore.pydispatch.Dispatcher",),
    name = _("Data root"),
    description = ""
)



# -*- coding: utf-8 -*-

import time
import unittest


from nive.utils.path import DvPath
from nive.definitions import ObjectConf, FieldConf, AppConf, DatabaseConf
from nive.portal import Portal
from nive_datastore.app import DataStorage


collection1 = ObjectConf("nive_datastore.item",
    id = "bookmark",
    name = "Bookmarks",
    dbparam = "bookmarks",
    subtypes="*",
    data = (
        FieldConf(id="link",     datatype="url",  size=500,   default="",  name="Link url"),
        FieldConf(id="share",    datatype="bool", size=2,     default=False,name="Share link"),
        FieldConf(id="comment",  datatype="text", size=50000, default="",  name="Comment"),
    ),
    forms = {
        "newItem": {"fields": ("link", "share", "comment"), "ajax":True, "newItem": True}, 
        "setItem": {"fields": ("link", "share", "comment"), "ajax":True}
    },
    toJson = ("id", "link", "comment", "pool_changedby", "pool_change"),
    template = "nive_datastore.webapi.tests:bookmark.pt"
)

collection2 = ObjectConf("nive_datastore.item",
    id = "track",
    name = "Track",
    dbparam = "tracks",
    subtypes=None,
    data = (
        FieldConf(id="url",       datatype="url",    size=500,   default="",  name="Url", required=True),
        FieldConf(id="number",    datatype="number", size=8,     default=0,    name="Some number"),
        FieldConf(id="something", datatype="text",   size=50000, default="",  name="Some text"),
    ),
    forms = {
        "newItem": {"fields": ("url", "number", "something"), "newItem": True }, 
        "setItem": {"fields": ("url", "number", "something") }
    },
    toJson = ("url", "number")
)


appconf = AppConf("nive_datastore.app",
    search = {
            "bookmarks":
                {"pool_type": "bookmark",
                 "container": False,
                 "fields": ["id", "link", "comment", "pool_changedby"],
                 "parameter": {},
                 "dynamic": {"size":10},
                 "size": 10},
            "tracks":
                {"pool_type": "track",
                 "container": True,
                 "fields": ["id", "link", "comment", "pool_changedby"],
                 "dynamic": {"size":10},
                 "parameter": {}},
            "default":
                {"container": False,
                 "fields": ["id", "pool_create", "pool_changedby"],
                 "dynamic": {"size":10, "start": 1},
                 "parameter": {}},
    },
)
appconf.modules.append(collection1)
appconf.modules.append(collection2)
appconf.lock()

def app_db(confs=None):
    a = DataStorage()
    a.Register(appconf)
    if confs:
        for c in confs:
            a.Register(c)
    p = Portal()
    p.Register(a)
    a.Startup(None)
    dbfile = DvPath(a.dbConfiguration.dbName)
    if not dbfile.IsFile():
        dbfile.CreateDirectories()
    try:
        a.Query("select id from pool_meta where id=1")
        a.Query("select pool_wfp from pool_meta where id=1")
        a.Query("select id from bookmarks where id=1")
        a.Query("select id from tracks where id=1")
        a.Query("select id from pool_files where id=1")
    except:
        a.GetTool("nive.tools.dbStructureUpdater")()
    return a

def app_nodb():
    a = DataStorage()
    a.Register(appconf)
    a.Register(DatabaseConf())
    p = Portal()
    p.Register(a)
    #a.Startup(None)
    return a

def create_bookmark(c, user):
    type = "bookmark"
    data = {"link": "the link", "comment": "some text"}
    return c.Create(type, data=data, user=user)

def create_track(c, user):
    type = "track"
    data = {"url": "the url", "number": 123, "something": "some text"}
    return c.Create(type, data=data, user=user)